@@ -48,6 +48,7 @@
  */
 
 #include <QCheckBox>
+#include <QFormLayout>
 #undef Below
 #undef Above
 #include <QSlider>
@@ -101,18 +102,18 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
     tabwidget = new QTabWidget(this);
     top->addWidget(tabwidget);
 
-    tab1 = new KMouseDlg(this);
-    QButtonGroup *group = new QButtonGroup( tab1 );
+    generalTab = new KMouseDlg(this);
+    QButtonGroup *group = new QButtonGroup( generalTab );
     group->setExclusive( true );
-    group->addButton( tab1->singleClick );
-    group->addButton( tab1->doubleClick );
+    group->addButton( generalTab->singleClick );
+    group->addButton( generalTab->doubleClick );
 
-    tabwidget->addTab(tab1, i18n("&General"));
+    tabwidget->addTab(generalTab, i18n("&General"));
 
-    group = new QButtonGroup( tab1 );
+    group = new QButtonGroup( generalTab );
     group->setExclusive( true );
-    group->addButton( tab1->rightHanded,RIGHT_HANDED );
-    group->addButton( tab1->leftHanded,LEFT_HANDED );
+    group->addButton( generalTab->rightHanded,RIGHT_HANDED );
+    group->addButton( generalTab->leftHanded,LEFT_HANDED );
 
     connect(group, SIGNAL(buttonClicked(int)), this, SLOT(changed()));
     connect(group, SIGNAL(buttonClicked(int)), this, SLOT(slotHandedChanged(int)));
@@ -123,9 +124,9 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
          " has more than two buttons, only those that function as the"
          " left and right buttons are affected. For example, if you have"
          " a three-button mouse, the middle button is unaffected.");
-    tab1->handedBox->setWhatsThis( wtstr );
+    generalTab->handedBox->setWhatsThis( wtstr );
 
-    connect(tab1->doubleClick, SIGNAL(clicked()), SLOT(changed()));
+    connect(generalTab->doubleClick, SIGNAL(clicked()), SLOT(changed()));
 
     wtstr = i18n("The default behavior in KDE is to select and activate"
          " icons with a single click of the left button on your pointing"
@@ -133,59 +134,59 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
          " when you click links in most web browsers. If you would prefer"
          " to select with a single click, and activate with a double click,"
          " check this option.");
-    tab1->doubleClick->setWhatsThis( wtstr );
+    generalTab->doubleClick->setWhatsThis( wtstr );
 
     wtstr = i18n("Activates and opens a file or folder with a single click.");
-    tab1->singleClick->setWhatsThis( wtstr );
+    generalTab->singleClick->setWhatsThis( wtstr );
 
 
-    connect(tab1->cbAutoSelect, SIGNAL(clicked()), this, SLOT(changed()));
+    connect(generalTab->cbAutoSelect, SIGNAL(clicked()), this, SLOT(changed()));
 
     wtstr = i18n("If you check this option, pausing the mouse pointer"
          " over an icon on the screen will automatically select that icon."
          " This may be useful when single clicks activate icons, and you"
          " want only to select the icon without activating it.");
-    tab1->cbAutoSelect->setWhatsThis( wtstr );
+    generalTab->cbAutoSelect->setWhatsThis( wtstr );
 
-//    slAutoSelect = new QSlider(0, 2000, 10, 0, QSlider::Horizontal, tab1);
-    tab1->slAutoSelect->setSingleStep( 125 );
-    tab1->slAutoSelect->setPageStep( 125 );
-    tab1->slAutoSelect->setTickPosition( QSlider::TicksBelow );
-    tab1->slAutoSelect->setTickInterval( 250 );
-    tab1->slAutoSelect->setTracking( true );
+//    slAutoSelect = new QSlider(0, 2000, 10, 0, QSlider::Horizontal, generalTab);
+    generalTab->slAutoSelect->setSingleStep( 125 );
+    generalTab->slAutoSelect->setPageStep( 125 );
+    generalTab->slAutoSelect->setTickPosition( QSlider::TicksBelow );
+    generalTab->slAutoSelect->setTickInterval( 250 );
+    generalTab->slAutoSelect->setTracking( true );
 
     wtstr = i18n("If you have checked the option to automatically select"
          " icons, this slider allows you to select how long the mouse pointer"
          " must be paused over the icon before it is selected.");
-    tab1->slAutoSelect->setWhatsThis( wtstr );
+    generalTab->slAutoSelect->setWhatsThis( wtstr );
 
-    connect(tab1->slAutoSelect, SIGNAL(valueChanged(int)), this, SLOT(changed()));
+    connect(generalTab->slAutoSelect, SIGNAL(valueChanged(int)), this, SLOT(changed()));
 
-    connect(tab1->cb_pointershape, SIGNAL(clicked()), this, SLOT(changed()));
+    connect(generalTab->cb_pointershape, SIGNAL(clicked()), this, SLOT(changed()));
 
-    connect(tab1->singleClick, SIGNAL(clicked()), this, SLOT(changed()));
-    connect(tab1->singleClick, SIGNAL(clicked()), this, SLOT(slotClick()));
-    connect(tab1->singleClick, SIGNAL(clicked()), this, SLOT(slotSmartSliderEnabling()));
+    connect(generalTab->singleClick, SIGNAL(clicked()), this, SLOT(changed()));
+    connect(generalTab->singleClick, SIGNAL(clicked()), this, SLOT(slotClick()));
+    connect(generalTab->singleClick, SIGNAL(clicked()), this, SLOT(slotSmartSliderEnabling()));
 
-    connect( tab1->doubleClick, SIGNAL( clicked() ), this, SLOT( slotClick() ) );
-    connect( tab1->cbAutoSelect, SIGNAL( clicked() ), this, SLOT( slotClick() ) );
-    connect(tab1->cbAutoSelect, SIGNAL(clicked()), this, SLOT(slotSmartSliderEnabling()));
+    connect( generalTab->doubleClick, SIGNAL( clicked() ), this, SLOT( slotClick() ) );
+    connect( generalTab->cbAutoSelect, SIGNAL( clicked() ), this, SLOT( slotClick() ) );
+    connect(generalTab->cbAutoSelect, SIGNAL(clicked()), this, SLOT(slotSmartSliderEnabling()));
 
 
     // Only allow setting reversing scroll polarity if we have scroll buttons
     unsigned char map[20];
     if ( XGetPointerMapping(QX11Info::display(), map, 20) >= 5 )
     {
-      tab1->cbScrollPolarity->setEnabled( true );
-      tab1->cbScrollPolarity->show();
+      generalTab->cbScrollPolarity->setEnabled( true );
+      generalTab->cbScrollPolarity->show();
     }
     else
     {
-      tab1->cbScrollPolarity->setEnabled( false );
-      tab1->cbScrollPolarity->hide();
+      generalTab->cbScrollPolarity->setEnabled( false );
+      generalTab->cbScrollPolarity->hide();
     }
-    connect(tab1->cbScrollPolarity, SIGNAL(clicked()), this, SLOT(changed()));
-    connect(tab1->cbScrollPolarity, SIGNAL(clicked()), this, SLOT(slotScrollPolarityChanged()));
+    connect(generalTab->cbScrollPolarity, SIGNAL(clicked()), this, SLOT(changed()));
+    connect(generalTab->cbScrollPolarity, SIGNAL(clicked()), this, SLOT(slotScrollPolarityChanged()));
 
     // Cursor theme tab
     themetab = new ThemePage(this);
@@ -193,18 +194,19 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
     tabwidget->addTab(themetab, i18n("&Cursor Theme"));
 
     // Advanced tab
-    tab2 = new QWidget(0);
-    tab2->setObjectName("Advanced Tab");
-    tabwidget->addTab(tab2, i18n("Advanced"));
+    advancedTab = new QWidget(0);
+    advancedTab->setObjectName("Advanced Tab");
+    tabwidget->addTab(advancedTab, i18n("Advanced"));
 
-    QBoxLayout *lay = new QVBoxLayout(tab2);
+    QFormLayout *lay = new QFormLayout(advancedTab);
     lay->setMargin(KDialog::marginHint());
     lay->setSpacing(KDialog::spacingHint());
 
-    accel = new KDoubleNumInput(0.1, 20, 2, tab2, 0.1, 1);
-    accel->setLabel(i18n("Pointer acceleration:"));
-    accel->setSuffix("x");
-    lay->addWidget(accel);
+    accel = new KDoubleNumInput(0.1, 20, 2, advancedTab, 0.1, 1);
+    accel->setSuffix(i18n(" x"));
+    QLabel* lbl = new QLabel(i18n("Pointer acceleration:"), advancedTab);
+    lbl->setBuddy(accel);
+    lay->addRow(lbl, accel);
     connect(accel, SIGNAL(valueChanged(double)), this, SLOT(changed()));
 
     wtstr = i18n("<p>This option allows you to change the relationship"
@@ -219,11 +221,12 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
          " flying across the screen, making it hard to control.</p>");
     accel->setWhatsThis( wtstr );
 
-    thresh = new KIntNumInput(20, tab2);
-    thresh->setLabel(i18n("Pointer threshold:"));
+    thresh = new KIntNumInput(20, advancedTab);
     thresh->setRange(0,20,1);
     thresh->setSteps(1,1);
-    lay->addWidget(thresh);
+    lbl = new QLabel(i18n("Pointer threshold:"), advancedTab);
+    lbl->setBuddy(thresh);
+    lay->addRow(lbl, thresh);
     connect(thresh, SIGNAL(valueChanged(int)), this, SLOT(changed()));
     connect(thresh, SIGNAL(valueChanged(int)), this, SLOT(slotThreshChanged(int)));
     slotThreshChanged(thresh->value());
@@ -241,12 +244,13 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
 
     // It would be nice if the user had a test field.
     // Selecting such values in milliseconds is not intuitive
-    doubleClickInterval = new KIntNumInput(2000, tab2);
-    doubleClickInterval->setLabel(i18n("Double click interval:"));
+    doubleClickInterval = new KIntNumInput(2000, advancedTab);
     doubleClickInterval->setRange(0, 2000, 100);
     doubleClickInterval->setSuffix(i18n(" msec"));
     doubleClickInterval->setSteps(100, 100);
-    lay->addWidget(doubleClickInterval);
+    lbl = new QLabel(i18n("Double click interval:"), advancedTab);
+    lbl->setBuddy(doubleClickInterval);
+    lay->addRow(lbl, doubleClickInterval);
     connect(doubleClickInterval, SIGNAL(valueChanged(int)), this, SLOT(changed()));
 
     wtstr = i18n("The double click interval is the maximal time"
@@ -257,14 +261,13 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
          " separate clicks.");
     doubleClickInterval->setWhatsThis( wtstr );
 
-    lay->addSpacing(15);
-
-    dragStartTime = new KIntNumInput(2000, tab2);
-    dragStartTime->setLabel(i18n("Drag start time:"));
+    dragStartTime = new KIntNumInput(2000, advancedTab);
     dragStartTime->setRange(0, 2000, 100);
     dragStartTime->setSuffix(i18n(" msec"));
     dragStartTime->setSteps(100, 100);
-    lay->addWidget(dragStartTime);
+    lbl = new QLabel(i18n("Drag start time:"), advancedTab);
+    lbl->setBuddy(dragStartTime);
+    lay->addRow(lbl, dragStartTime);
     connect(dragStartTime, SIGNAL(valueChanged(int)), this, SLOT(changed()));
 
     wtstr = i18n("If you click with the mouse (e.g. in a multi-line"
@@ -272,11 +275,12 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
          " drag start time, a drag operation will be initiated.");
     dragStartTime->setWhatsThis( wtstr );
 
-    dragStartDist = new KIntNumInput(20, tab2);
-    dragStartDist->setLabel(i18n("Drag start distance:"));
+    dragStartDist = new KIntNumInput(20, advancedTab);
     dragStartDist->setRange(1, 20, 1);
     dragStartDist->setSteps(1,1);
-    lay->addWidget(dragStartDist);
+    lbl = new QLabel(i18n("Drag start distance:"), advancedTab);
+    lbl->setBuddy(dragStartDist);
+    lay->addRow(lbl, dragStartDist);
     connect(dragStartDist, SIGNAL(valueChanged(int)), this, SLOT(changed()));
     connect(dragStartDist, SIGNAL(valueChanged(int)), this, SLOT(slotDragStartDistChanged(int)));
     slotDragStartDistChanged(dragStartDist->value());
@@ -286,18 +290,18 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
          " operation will be initiated.");
     dragStartDist->setWhatsThis( wtstr );
 
-    wheelScrollLines = new KIntNumInput(3, tab2);
-    wheelScrollLines->setLabel(i18n("Mouse wheel scrolls by:"));
+    wheelScrollLines = new KIntNumInput(3, advancedTab);
     wheelScrollLines->setRange(1, 12, 1);
     wheelScrollLines->setSteps(1,1);
-    lay->addWidget(wheelScrollLines);
+    lbl = new QLabel(i18n("Mouse wheel scrolls by:"), advancedTab);
+    lbl->setBuddy(wheelScrollLines);
+    lay->addRow(lbl, wheelScrollLines);
     connect(wheelScrollLines, SIGNAL(valueChanged(int)), this, SLOT(changed()));
     connect(wheelScrollLines, SIGNAL(valueChanged(int)), SLOT(slotWheelScrollLinesChanged(int)));
     slotWheelScrollLinesChanged(wheelScrollLines->value());
 
     wtstr = i18n("If you use the wheel of a mouse, this value determines the number of lines to scroll for each wheel movement. Note that if this number exceeds the number of visible lines, it will be ignored and the wheel movement will be handled as a page up/down movement.");
     wheelScrollLines->setWhatsThis(wtstr);
-    lay->addStretch();
 
 {
   QWidget *mouse = new QWidget(this);
@@ -315,54 +319,43 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
   mouseKeys = new QCheckBox(i18n("&Move pointer with keyboard (using the num pad)"), mouse);
   vvbox->addWidget(mouseKeys);
 
-  QBoxLayout *hbox = new QHBoxLayout();
-  vvbox->addItem( hbox );
-  hbox->setSpacing(KDialog::spacingHint());
-  hbox->addSpacing(24);
+  QFormLayout *form = new QFormLayout();
+  vvbox->addItem( form );
+  form->setSpacing(KDialog::spacingHint());
+
   mk_delay = new KIntNumInput(mouse);
-  mk_delay->setLabel(i18n("&Acceleration delay:"), Qt::AlignVCenter);
   mk_delay->setSuffix(i18n(" msec"));
   mk_delay->setRange(1, 1000, 50);
-  hbox->addWidget(mk_delay);
+  lbl = new QLabel(i18n("&Acceleration delay:"), mouse);
+  lbl->setBuddy(mk_delay);
+  form->addRow(lbl, mk_delay);
 
-  hbox = new QHBoxLayout();
-  vvbox->addItem(hbox);
-  hbox->setSpacing(KDialog::spacingHint());
-  hbox->addSpacing(24);
   mk_interval = new KIntNumInput(0, mouse);
-  mk_interval->setLabel(i18n("R&epeat interval:"), Qt::AlignVCenter);
   mk_interval->setSuffix(i18n(" msec"));
   mk_interval->setRange(1, 1000, 10);
-  hbox->addWidget(mk_interval);
+  lbl = new QLabel(i18n("R&epeat interval:"), mouse);
+  lbl->setBuddy(mk_interval);
+  form->addRow(lbl, mk_interval);
 
-  hbox = new QHBoxLayout();
-  vvbox->addItem(hbox);
-  hbox->setSpacing(KDialog::spacingHint());
-  hbox->addSpacing(24);
   mk_time_to_max = new KIntNumInput(0, mouse);
-  mk_time_to_max->setLabel(i18n("Acceleration &time:"), Qt::AlignVCenter);
   mk_time_to_max->setRange(100, 10000, 200);
   mk_time_to_max->setSuffix(i18n(" msec"));
-  hbox->addWidget(mk_time_to_max);
+  lbl = new QLabel(i18n("Acceleration &time:"), mouse);
+  lbl->setBuddy(mk_time_to_max);
+  form->addRow(lbl, mk_time_to_max);
 
-  hbox = new QHBoxLayout();
-  vvbox->addItem(hbox);
-  hbox->setSpacing(KDialog::spacingHint());
-  hbox->addSpacing(24);
   mk_max_speed = new KIntNumInput(0, mouse);
-  mk_max_speed->setLabel(i18n("Ma&ximum speed:"), Qt::AlignVCenter);
   mk_max_speed->setRange(1, 2000, 20);
   mk_max_speed->setSuffix(i18n(" pixel/sec"));
-  hbox->addWidget(mk_max_speed);
+  lbl = new QLabel(i18n("Ma&ximum speed:"), mouse);
+  lbl->setBuddy(mk_max_speed);
+  form->addRow(lbl, mk_max_speed);
 
-  hbox = new QHBoxLayout();
-  vvbox->addItem(hbox);
-  hbox->setSpacing(KDialog::spacingHint());
-  hbox->addSpacing(24);
   mk_curve = new KIntNumInput(0, mouse);
-  mk_curve->setLabel(i18n("Acceleration &profile:"), Qt::AlignVCenter);
   mk_curve->setRange(-1000, 1000, 100);
-  hbox->addWidget(mk_curve);
+  lbl = new QLabel(i18n("Acceleration &profile:"), mouse);
+  lbl->setBuddy(mk_curve);
+  form->addRow(lbl, mk_curve);
 
   connect(mouseKeys, SIGNAL(clicked()), this, SLOT(checkAccess()));
   connect(mouseKeys, SIGNAL(clicked()), this, SLOT(changed()));
@@ -381,11 +374,12 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
   // It only works if libusb is available.
 #ifdef HAVE_LIBUSB
 
+  static const
   struct device_table {
       int idVendor;
       int idProduct;
-      QString Model;
-      QString Name;
+      const char* Model;
+      const char* Name;
       int flags;
   } device_table[] = {
       { VENDOR_LOGITECH, 0xC00E, "M-BJ58", "Wheel Mouse Optical", HAS_RES },
@@ -405,7 +399,7 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
       { VENDOR_LOGITECH, 0xC50B, "967300-0403", "Cordless MX Duo Receiver", HAS_SS|HAS_CSR },
       { VENDOR_LOGITECH, 0xC50E, "M-RAG97", "MX1000 Laser Mouse", HAS_SS | HAS_CSR },
       { VENDOR_LOGITECH, 0xC702, "C-UF15", "Receiver for Cordless Presenter", HAS_CSR },
-      { 0, 0, QString(), QString(), 0 }
+      { 0, 0, 0, 0, 0 }
   };
 
   usb_init();
@@ -421,7 +415,7 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
 	      if ( (device_table[n].idVendor == dev->descriptor.idVendor) &&
 		   (device_table[n].idProduct == dev->descriptor.idProduct) ) {
 		  // OK, we have a device that appears to be one of the ones we support
-		  LogitechMouse *mouse = new LogitechMouse( dev, device_table[n].flags, this, device_table[n].Name.toLatin1() );
+		  LogitechMouse *mouse = new LogitechMouse( dev, device_table[n].flags, this, device_table[n].Name );
 		  settings->logitechMouseList.append(mouse);
 		  tabwidget->addTab( (QWidget*)mouse, device_table[n].Name );
 	      }
@@ -430,8 +424,6 @@ MouseConfig::MouseConfig(QWidget *parent, const QVariantList &args)
 
 #endif
 
-  load();
-
   KAboutData* about = new KAboutData("kcmmouse", 0, ki18n("Mouse"), 0, KLocalizedString(),
         KAboutData::License_GPL, ki18n("(c) 1997 - 2005 Mouse developers"));
   about->addAuthor(ki18n("Patrick Dowler"));
@@ -483,7 +475,7 @@ void MouseConfig::setThreshold(int val)
 
 int MouseConfig::getHandedness()
 {
-  if (tab1->rightHanded->isChecked())
+  if (generalTab->rightHanded->isChecked())
     return RIGHT_HANDED;
   else
     return LEFT_HANDED;
@@ -491,16 +483,17 @@ int MouseConfig::getHandedness()
 
 void MouseConfig::setHandedness(int val)
 {
-  tab1->rightHanded->setChecked(false);
-  tab1->leftHanded->setChecked(false);
+  generalTab->rightHanded->setChecked(false);
+  generalTab->leftHanded->setChecked(false);
   if (val == RIGHT_HANDED){
-    tab1->rightHanded->setChecked(true);
-    tab1->mousePix->setPixmap(KStandardDirs::locate("data", "kcminput/pics/mouse_rh.png"));
+    generalTab->rightHanded->setChecked(true);
+    generalTab->mousePix->setPixmap(KStandardDirs::locate("data", "kcminput/pics/mouse_rh.png"));
   }
   else{
-    tab1->leftHanded->setChecked(true);
-    tab1->mousePix->setPixmap(KStandardDirs::locate("data", "kcminput/pics/mouse_lh.png"));
+    generalTab->leftHanded->setChecked(true);
+    generalTab->mousePix->setPixmap(KStandardDirs::locate("data", "kcminput/pics/mouse_lh.png"));
   }
+  settings->m_handedNeedsApply = true;
 }
 
 void MouseConfig::load()
@@ -508,11 +501,11 @@ void MouseConfig::load()
   KConfig config( "kcminputrc" );
   settings->load(&config);
 
-  tab1->rightHanded->setEnabled(settings->handedEnabled);
-  tab1->leftHanded->setEnabled(settings->handedEnabled);
-  if ( tab1->cbScrollPolarity->isEnabled() )
-    tab1->cbScrollPolarity->setEnabled(settings->handedEnabled);
-  tab1->cbScrollPolarity->setChecked( settings->reverseScrollPolarity );
+  generalTab->rightHanded->setEnabled(settings->handedEnabled);
+  generalTab->leftHanded->setEnabled(settings->handedEnabled);
+  if ( generalTab->cbScrollPolarity->isEnabled() )
+    generalTab->cbScrollPolarity->setEnabled(settings->handedEnabled);
+  generalTab->cbScrollPolarity->setChecked( settings->reverseScrollPolarity );
 
   setAccel(settings->accelRate);
   setThreshold(settings->thresholdMove);
@@ -523,14 +516,14 @@ void MouseConfig::load()
   dragStartDist->setValue(settings->dragStartDist);
   wheelScrollLines->setValue(settings->wheelScrollLines);
 
-  tab1->singleClick->setChecked( settings->singleClick );
-  tab1->doubleClick->setChecked(!settings->singleClick);
-  tab1->cb_pointershape->setChecked(settings->changeCursor);
-  tab1->cbAutoSelect->setChecked( settings->autoSelectDelay >= 0 );
+  generalTab->singleClick->setChecked( settings->singleClick );
+  generalTab->doubleClick->setChecked(!settings->singleClick);
+  generalTab->cb_pointershape->setChecked(settings->changeCursor);
+  generalTab->cbAutoSelect->setChecked( settings->autoSelectDelay >= 0 );
   if ( settings->autoSelectDelay < 0 )
-     tab1->slAutoSelect->setValue( 0 );
+     generalTab->slAutoSelect->setValue( 0 );
   else
-     tab1->slAutoSelect->setValue( settings->autoSelectDelay );
+     generalTab->slAutoSelect->setValue( settings->autoSelectDelay );
   slotClick();
 
 
@@ -563,7 +556,8 @@ void MouseConfig::load()
   themetab->load();
 
   checkAccess();
-  changed();
+
+  emit changed(false);
 }
 
 void MouseConfig::save()
@@ -576,11 +570,11 @@ void MouseConfig::save()
   settings->dragStartTime = dragStartTime->value();
   settings->dragStartDist = dragStartDist->value();
   settings->wheelScrollLines = wheelScrollLines->value();
-  settings->singleClick = !tab1->doubleClick->isChecked();
-  settings->autoSelectDelay = tab1->cbAutoSelect->isChecked()? tab1->slAutoSelect->value():-1;
-//  settings->changeCursor = tab1->singleClick->isChecked();
-  settings->changeCursor = tab1->cb_pointershape->isChecked();
-  settings->reverseScrollPolarity = tab1->cbScrollPolarity->isChecked();
+  settings->singleClick = !generalTab->doubleClick->isChecked();
+  settings->autoSelectDelay = generalTab->cbAutoSelect->isChecked()? generalTab->slAutoSelect->value():-1;
+//  settings->changeCursor = generalTab->singleClick->isChecked();
+  settings->changeCursor = generalTab->cb_pointershape->isChecked();
+  settings->reverseScrollPolarity = generalTab->cbScrollPolarity->isChecked();
 
   settings->apply();
   KConfig config( "kcminputrc" );
@@ -609,8 +603,7 @@ void MouseConfig::save()
   // restart kaccess
   KToolInvocation::startServiceByDesktopName("kaccess");
 
-  KCModule::changed(false);
-
+  emit changed(false);
 }
 
 void MouseConfig::defaults()
@@ -618,16 +611,16 @@ void MouseConfig::defaults()
     setThreshold(2);
     setAccel(2);
     setHandedness(RIGHT_HANDED);
-    tab1->cbScrollPolarity->setChecked( false );
+    generalTab->cbScrollPolarity->setChecked( false );
     doubleClickInterval->setValue(400);
     dragStartTime->setValue(500);
     dragStartDist->setValue(4);
     wheelScrollLines->setValue(3);
-    tab1->doubleClick->setChecked( !KDE_DEFAULT_SINGLECLICK );
-    tab1->cbAutoSelect->setChecked( KDE_DEFAULT_AUTOSELECTDELAY != -1 );
-    tab1->slAutoSelect->setValue( KDE_DEFAULT_AUTOSELECTDELAY == -1 ? 50 : KDE_DEFAULT_AUTOSELECTDELAY );
-    tab1->singleClick->setChecked( KDE_DEFAULT_SINGLECLICK );
-    tab1->cb_pointershape->setChecked(KDE_DEFAULT_CHANGECURSOR);
+    generalTab->doubleClick->setChecked( !KDE_DEFAULT_SINGLECLICK );
+    generalTab->cbAutoSelect->setChecked( KDE_DEFAULT_AUTOSELECTDELAY != -1 );
+    generalTab->slAutoSelect->setValue( KDE_DEFAULT_AUTOSELECTDELAY == -1 ? 50 : KDE_DEFAULT_AUTOSELECTDELAY );
+    generalTab->singleClick->setChecked( KDE_DEFAULT_SINGLECLICK );
+    generalTab->cb_pointershape->setChecked(KDE_DEFAULT_CHANGECURSOR);
     slotClick();
 
   mouseKeys->setChecked(false);
@@ -636,6 +629,8 @@ void MouseConfig::defaults()
   mk_time_to_max->setValue(5000);
   mk_max_speed->setValue(1000);
   mk_curve->setValue(0);
+  
+  themetab->defaults();
 
   checkAccess();
 
@@ -645,22 +640,22 @@ void MouseConfig::defaults()
 void MouseConfig::slotClick()
 {
   // Autoselect has a meaning only in single-click mode
-  tab1->cbAutoSelect->setEnabled(!tab1->doubleClick->isChecked() || tab1->singleClick->isChecked());
+  generalTab->cbAutoSelect->setEnabled(!generalTab->doubleClick->isChecked() || generalTab->singleClick->isChecked());
   // Delay has a meaning only for autoselect
-  bool bDelay = tab1->cbAutoSelect->isChecked() && ! tab1->doubleClick->isChecked();
-   tab1->slAutoSelect->setEnabled( bDelay );
-   tab1->lDelay->setEnabled( bDelay );
-   tab1->lb_short->setEnabled( bDelay );
-   tab1->lb_long->setEnabled( bDelay );
+  bool bDelay = generalTab->cbAutoSelect->isChecked() && ! generalTab->doubleClick->isChecked();
+   generalTab->slAutoSelect->setEnabled( bDelay );
+   generalTab->lDelay->setEnabled( bDelay );
+   generalTab->lb_short->setEnabled( bDelay );
+   generalTab->lb_long->setEnabled( bDelay );
 
 }
 
 /** No descriptions */
 void MouseConfig::slotHandedChanged(int val){
   if(val==RIGHT_HANDED)
-    tab1->mousePix->setPixmap(KStandardDirs::locate("data", "kcminput/pics/mouse_rh.png"));
+    generalTab->mousePix->setPixmap(KStandardDirs::locate("data", "kcminput/pics/mouse_rh.png"));
   else
-    tab1->mousePix->setPixmap(KStandardDirs::locate("data", "kcminput/pics/mouse_lh.png"));
+    generalTab->mousePix->setPixmap(KStandardDirs::locate("data", "kcminput/pics/mouse_lh.png"));
   settings->m_handedNeedsApply = true;
 }
 
@@ -710,7 +705,7 @@ void MouseSettings::load(KConfig *config)
   }
 
   KConfigGroup group = config->group("Mouse");
-  double a = group.readEntry("Acceleration",-1);
+  double a = group.readEntry("Acceleration",-1.0);
   if (a == -1)
     accelRate = accel;
   else
@@ -878,11 +873,11 @@ void MouseConfig::slotScrollPolarityChanged()
 
 void MouseConfig::slotSmartSliderEnabling()
 {
-  bool enabled = tab1->singleClick->isChecked() ? tab1->cbAutoSelect->isChecked() : false;
-  tab1->lDelay->setEnabled(enabled);
-  tab1->slAutoSelect->setEnabled(enabled);
-  tab1->lb_short->setEnabled(enabled);
-  tab1->lb_long->setEnabled(enabled);
+  bool enabled = generalTab->singleClick->isChecked() ? generalTab->cbAutoSelect->isChecked() : false;
+  generalTab->lDelay->setEnabled(enabled);
+  generalTab->slAutoSelect->setEnabled(enabled);
+  generalTab->lb_short->setEnabled(enabled);
+  generalTab->lb_long->setEnabled(enabled);
 }
 
 #include "mouse.moc"
