@@ -75,6 +75,7 @@ ThemePage::ThemePage(QWidget *parent)
     view->setModel(proxy);
     view->setItemDelegate(new ItemDelegate(this));
     view->setIconSize(QSize(size, size));
+    view->setSelectionMode( QAbstractItemView::SingleSelection );
 
     // Make sure we find out about selection changes
     connect(view->selectionModel(),
@@ -188,6 +189,7 @@ void ThemePage::save()
     KConfig config("kcminputrc");
     KConfigGroup c(&config, "Mouse");
     c.writeEntry("cursorTheme", theme->name());
+    c.sync();
 
     if (!applyTheme(theme))
     {
@@ -202,6 +204,7 @@ void ThemePage::save()
 
 void ThemePage::load()
 {
+    view->selectionModel()->clear();
     // Get the name of the theme libXcursor currently uses
     QString currentTheme = XcursorGetTheme(x11Info().display());
 
@@ -210,34 +213,42 @@ void ThemePage::load()
     KConfigGroup cg(&c, "Mouse");
     currentTheme = cg.readEntry("cursorTheme", currentTheme);
 
-    // Find the theme in the listview and select it
+    // Find the theme in the listview
     if (!currentTheme.isEmpty())
         appliedIndex = proxy->findIndex(currentTheme);
     else
         appliedIndex = proxy->defaultIndex();
 
-    selectRow(appliedIndex);
-    view->scrollTo(appliedIndex, QListView::PositionAtCenter);
-
-    // Update the preview widget as well
-    const CursorTheme *theme = proxy->theme(appliedIndex);
-    preview->setTheme(theme);
-
     // Disable the listview and the buttons if we're in kiosk mode
-    if (cg.entryIsImmutable("cursorTheme"))
+    if (cg.isEntryImmutable("cursorTheme"))
     {
         view->setEnabled(false);
         installButton->setEnabled(false);
         removeButton->setEnabled(false);
     }
 
-    if (!theme->isWritable())
+    const CursorTheme *theme = proxy->theme(appliedIndex);
+
+    if (appliedIndex.isValid())
+    {
+        // Select the current theme
+        selectRow(appliedIndex);
+        view->scrollTo(appliedIndex, QListView::PositionAtCenter);
+
+        // Update the preview widget as well
+        preview->setTheme(theme);
+    }
+
+    if (!theme || !theme->isWritable())
         removeButton->setEnabled(false);
 }
 
 
 void ThemePage::defaults()
 {
+    view->selectionModel()->clear();
+    QModelIndex defaultIndex = proxy->findIndex("Oxygen_Black");
+    view->setCurrentIndex(defaultIndex);
 }
 
 
