@@ -28,7 +28,7 @@ namespace KWin
 
 class Client;
 
-class Bridge : public KDecorationBridge
+class Bridge : public KDecorationBridgeUnstable
     {
     public:
         Bridge( Client* cl );
@@ -74,6 +74,8 @@ class Bridge : public KDecorationBridge
         virtual QWidget* initialParentWidget() const;
         virtual Qt::WFlags initialWFlags() const;
         virtual void grabXServer( bool grab );
+
+        virtual bool compositingActive() const;
     private:
         Client* c;
     };
