@@ -32,6 +32,8 @@
 #include <QIcon>
 #include <QPixmap>
 #include <QList>
+#include <QMimeData>
+#include <QTextDocument>
 
 #define HEADING_X_ICON_SIZE 16
 
@@ -63,13 +65,16 @@ ProcessModelPrivate::ProcessModelPrivate() :  mBlankPixmap(HEADING_X_ICON_SIZE,1
 	mProcesses = NULL;
 	mShowChildTotals = true;
 	mIsChangingLayout = false;
+	mShowCommandLineOptions = false;
+	mShowingTooltips = true;
+	mNormalizeCPUUsage = true;
 }
 
 ProcessModelPrivate::~ProcessModelPrivate()
 {
 	if(mProcesses)
 		KSysGuard::Processes::returnInstance(mHostName);
-	foreach(WindowInfo wininfo, mPidToWindowInfo) {
+	foreach(const WindowInfo &wininfo, mPidToWindowInfo) {
 		delete wininfo.netWinInfo;
 	}
 	mProcesses = NULL;
@@ -104,11 +109,11 @@ KSysGuard::Processes *ProcessModel::processController()
 
 void ProcessModelPrivate::windowRemoved(WId wid) {
 #ifdef Q_WS_X11
-	long long pid = mWIdToPid.value(wid, 0);
+	qlonglong pid = mWIdToPid.value(wid, 0);
 	if(pid <= 0) return;
 
 	int count = mPidToWindowInfo.count(pid);
-	QMultiHash<long long, WindowInfo>::iterator i = mPidToWindowInfo.find(pid);
+	QMultiHash<qlonglong, WindowInfo>::iterator i = mPidToWindowInfo.find(pid);
 	while (i != mPidToWindowInfo.end() && i.key() == pid) {
 		if(i.value().wid == wid) {
 			delete i.value().netWinInfo;
@@ -178,7 +183,7 @@ void ProcessModelPrivate::windowChanged(WId wid, unsigned int properties)
 
 void ProcessModelPrivate::windowAdded(WId wid)
 {
-	foreach(WindowInfo w, mPidToWindowInfo.values()) {
+	foreach(const WindowInfo &w, mPidToWindowInfo) {
 		if(w.wid == wid) return; //already added
 	}
 	//The name changed
@@ -189,7 +194,7 @@ void ProcessModelPrivate::windowAdded(WId wid)
 		delete info;
 		return;  //info is invalid - window just closed or something probably
         }
-	long long pid = info->pid();
+	qlonglong pid = info->pid();
 	if(pid <= 0) {
 		delete info;
 		return;
@@ -232,13 +237,13 @@ QString ProcessModelPrivate::getStatusDescription(KSysGuard::Process::ProcessSta
 {
 	switch( status) {
 		case KSysGuard::Process::Running:
-			return i18n("- Process is doing some work");
+			return i18n("- Process is doing some work.");
 		case KSysGuard::Process::Sleeping:
-			return i18n("- Process is waiting for something to happen");
+			return i18n("- Process is waiting for something to happen.");
 		case KSysGuard::Process::Stopped:
-			return i18n("- Process has been stopped. It will not respond to user input at the moment");
+			return i18n("- Process has been stopped. It will not respond to user input at the moment.");
 		case KSysGuard::Process::Zombie:
-			return i18n("- Process has finished and is now dead, but the parent process has not cleaned up");
+			return i18n("- Process has finished and is now dead, but the parent process has not cleaned up.");
 		default:
 			return QString();
 	}
@@ -532,29 +537,40 @@ QVariant ProcessModel::headerData(int section, Qt::Orientation orientation,
 	  }
 	  case Qt::ToolTipRole: 
 	  {
+                if(!d->mShowingTooltips)
+		    return QVariant();
 		switch(section) {
 		    case HeadingName:
-			return i18n("The process name");
+			return i18n("The process name.");
 		    case HeadingUser:
-			return i18n("The user that owns this process");
+			return i18n("The user who owns this process.");
 		    case HeadingTty:
-			return i18n("The controlling terminal that this process is running on.");
+			return i18n("The controlling terminal on which this process is running.");
 		    case HeadingNiceness:
-			return i18n("The priority that this process is being run with. Ranges from 19 (very nice, least priority) to -19 (top priority)");
+			return i18n("The priority with which this process is being run. Ranges from 19 (very nice, least priority) to -19 (top priority).");
 		    case HeadingCPUUsage:
-			return i18n("The current CPU usage of the process, divided by the number of processor cores in the machine.");
+			if(d->mNumProcessorCores == 1)
+			    return i18n("The current CPU usage of the process.");
+			else
+			    // i18n: %1 is always greater than 1, so do not worry about
+			    // nonsensical verbosity of the singular part.
+			    if(d->mNormalizeCPUUsage)
+				    return i18np("The current total CPU usage of the process, divided by the %1 processor core in the machine.", "The current total CPU usage of the process, divided by the %1 processor cores in the machine.", d->mNumProcessorCores);
+			    else
+				    return i18n("The current total CPU usage of the process.");
+				    
 		    case HeadingVmSize:
 			return i18n("<qt>This is the amount of virtual memory space that the process is using, included shared libraries, graphics memory, files on disk, and so on. This number is almost meaningless.</qt>");
 		    case HeadingMemory:
-			return i18n("<qt>This is the amount of real physical memory that this process is using by itself. It does not include any swapped out memory, nor the code size of its shared libraries. This is often the most useful figure to judge the memory use of a program.</qt>");
+			return i18n("<qt>This is the amount of real physical memory that this process is using by itself.<br>It does not include any swapped out memory, nor the code size of its shared libraries.<br>This is often the most useful figure to judge the memory use of a program.</qt>");
 		    case HeadingSharedMemory:
-			return i18n("<qt>This is the amount of real physical memory that this process's shared libraries are using. This memory is shared among all processes that use this library</qt>");
+			return i18n("<qt>This is the amount of real physical memory that this process's shared libraries are using.<br>This memory is shared among all processes that use this library.</qt>");
 		    case HeadingCommand:
-			return i18n("<qt>The command that this process was launched with</qt>");
+			return i18n("<qt>The command with which this process was launched.</qt>");
 		    case HeadingXTitle:
-			return i18n("<qt>The title of any windows that this process is showing</qt>");
+			return i18n("<qt>The title of any windows that this process is showing.</qt>");
 		    case HeadingPid:
-			return i18n("The unique Process ID that identifies this process");
+			return i18n("The unique Process ID that identifies this process.");
 		    default:
 			return QVariant();
 		}
@@ -599,7 +615,7 @@ void ProcessModel::setSimpleMode(bool simple)
 
 }
 
-bool ProcessModel::canUserLogin(long long uid ) const
+bool ProcessModel::canUserLogin(long uid ) const
 {
 	if(uid == 65534) {
 		//nobody user
@@ -632,20 +648,21 @@ bool ProcessModel::canUserLogin(long long uid ) const
 
 QString ProcessModelPrivate::getTooltipForUser(const KSysGuard::Process *ps) const
 {
-	QString userTooltip;
+	QString userTooltip = "<qt><p style='white-space:pre'>";
 	if(!mIsLocalhost) {
-		userTooltip = "<qt>";
 		userTooltip += i18n("Login Name: %1<br/>", getUsernameForUser(ps->uid, true));
 	} else {
 		KUser user(ps->uid);
 		if(!user.isValid())
-			userTooltip = i18n("This user is not recognised for some reason");
+			userTooltip += i18n("This user is not recognized for some reason.");
 		else {
-			userTooltip = "<qt>";
-            if(!user.property(KUser::FullName).isValid()) userTooltip += i18n("<b>%1</b><br/>", user.property(KUser::FullName).toString());
+			if(!user.property(KUser::FullName).isValid()) 
+				userTooltip += i18n("<b>%1</b><br/>", user.property(KUser::FullName).toString());
 			userTooltip += i18n("Login Name: %1 (uid: %2)<br/>", user.loginName(), ps->uid);
-            if(!user.property(KUser::RoomNumber).isValid()) userTooltip += i18n("  Room Number: %1<br/>", user.property(KUser::RoomNumber).toString());
-            if(!user.property(KUser::WorkPhone).isValid()) userTooltip += i18n("  Work Phone: %1<br/>", user.property(KUser::WorkPhone).toString());
+			if(!user.property(KUser::RoomNumber).isValid()) 
+				userTooltip += i18n("  Room Number: %1<br/>", user.property(KUser::RoomNumber).toString());
+			if(!user.property(KUser::WorkPhone).isValid()) 
+				userTooltip += i18n("  Work Phone: %1<br/>", user.property(KUser::WorkPhone).toString());
 		}
 	}
 	if( (ps->uid != ps->euid && ps->euid != -1) || 
@@ -679,7 +696,7 @@ QString ProcessModel::getStringForProcess(KSysGuard::Process *process) const {
 	return i18nc("Short description of a process. PID, name, user", "<numid>%1</numid>: %2, owned by user %3", (long)(process->pid), process->name, d->getUsernameForUser(process->uid, false));
 }
 
-QString ProcessModelPrivate::getGroupnameForGroup(long long gid) const {
+QString ProcessModelPrivate::getGroupnameForGroup(long gid) const {
 	if(mIsLocalhost) {
 		QString groupname = KUserGroup(gid).name();
 		if(!groupname.isEmpty())
@@ -688,7 +705,7 @@ QString ProcessModelPrivate::getGroupnameForGroup(long long gid) const {
 	return QString::number(gid);
 }
 
-QString ProcessModelPrivate::getUsernameForUser(long long uid, bool withuid) const {
+QString ProcessModelPrivate::getUsernameForUser(long uid, bool withuid) const {
 	QString &username = mUserUsername[uid];
 	if(username.isNull()) {
 		if(!mIsLocalhost) {
@@ -725,7 +742,10 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 		KSysGuard::Process *process = reinterpret_cast< KSysGuard::Process * > (index.internalPointer());
 		switch(index.column()) {
 		case HeadingName:
-			return process->name;
+			if(d->mShowCommandLineOptions)
+				return process->name;
+			else
+				return process->name.section(' ', 0,0);
 		case HeadingPid:
 			return (qlonglong)process->pid;
 		case HeadingUser:
@@ -743,7 +763,8 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 				double total;
 				if(d->mShowChildTotals && !d->mSimple) total = process->totalUserUsage + process->totalSysUsage;
 				else total = process->userUsage + process->sysUsage;
-				total = total / d->mNumProcessorCores;
+				if(d->mNormalizeCPUUsage)
+					total = total / d->mNumProcessorCores;
 
 				if(total < 1 && process->status != KSysGuard::Process::Sleeping && process->status != KSysGuard::Process::Running)
 					return process->translatedStatus();  //tell the user when the process is a zombie or stopped
@@ -795,6 +816,8 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 		break;
 	}
 	case Qt::ToolTipRole: {
+                if(!d->mShowingTooltips)
+		    return QVariant();
 		KSysGuard::Process *process = reinterpret_cast< KSysGuard::Process * > (index.internalPointer());
 		QString tracer;
 		if(process->tracerpid > 0) {
@@ -806,9 +829,9 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 		}
 		switch(index.column()) {
 		case HeadingName: {
-			QString tooltip = "<qt>";
-			/***  It would be nice to be able to show the icon in the tooltip, but Qt4 won't let us put
-			 *    a picture in a tooltip :(
+			QString tooltip = "<qt><p style='white-space:pre'>";
+			/*   It would be nice to be able to show the icon in the tooltip, but Qt4 won't let us put
+			 *   a picture in a tooltip :(
 			   
 			QIcon icon;
 			if(mPidToWindowInfo.contains(process->pid)) {
@@ -825,12 +848,21 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 				if(process->name == "init") {
 					tooltip += i18n("<b>Init</b> is the parent of all other processes and cannot be killed.<br/>");
 				} else if(process->name == "kthreadd") {
-					tooltip += i18n("<b>KThreadd</b> manages kernel threads. The children processes run in the kernel, controlling hard disk access etc.<br/>");
+					tooltip += i18n("<b>KThreadd</b> manages kernel threads. The children processes run in the kernel, controlling hard disk access, etc.<br/>");
 				}
-				tooltip	+= i18nc("name column tooltip. first item is the name","<b>%1</b><br />Process ID: <numid>%2</numid><br />Command: %3", process->name, (long int)process->pid, process->command);
+				tooltip	+= i18nc("name column tooltip. first item is the name","<b>%1</b><br />Process ID: <numid>%2</numid>", process->name, (long int)process->pid);
+			}
+			else {
+				KSysGuard::Process *parent_process = d->mProcesses->getProcess(process->parent_pid);
+				if(parent_process) { //it should not be possible for this process to not exist, but check just incase
+					tooltip	= i18nc("name column tooltip. first item is the name","<b>%1</b><br />Process ID: <numid>%2</numid><br />Parent: %3<br />Parent's ID: <numid>%4</numid>", process->name, (long int)process->pid, parent_process->name, (long int)process->parent_pid);
+				} else {
+					tooltip	= i18nc("name column tooltip. first item is the name","<b>%1</b><br />Process ID: <numid>%2</numid><br />Parent's ID: <numid>%3</numid>", process->name, (long int)process->pid, (long int)process->parent_pid);
+				}
+			}
+			if(!process->command.isEmpty()) {
+				tooltip+= i18n("<br/>Command: %1", process->command);
 			}
-			else
-				tooltip	= i18nc("name column tooltip. first item is the name","<b>%1</b><br />Process ID: <numid>%2</numid><br />Parent's ID: <numid>%3</numid><br />Command: %4", process->name, (long int)process->pid, (long int)process->parent_pid, process->command);
 			if(!process->tty.isEmpty())
 				tooltip += i18n( "<br />Running on: %1", QString(process->tty));
 			if(!tracer.isEmpty())
@@ -852,15 +884,15 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 			return d->getTooltipForUser(process);
 		}
 		case HeadingNiceness: {
-		        QString tooltip;
+		        QString tooltip = "<qt><p style='white-space:pre'>";
 			switch(process->scheduler) {
 			  case KSysGuard::Process::Other:
 			  case KSysGuard::Process::Batch:
-				  tooltip = i18n("<qt>Nice level: %1 (%2)", process->niceLevel, process->niceLevelAsString() );
+				  tooltip += i18n("Nice level: %1 (%2)", process->niceLevel, process->niceLevelAsString() );
 				  break;
 			  case KSysGuard::Process::RoundRobin:
 			  case KSysGuard::Process::Fifo:
-				  tooltip = i18n("<qt>Scheduler priority: %1", process->niceLevel);
+				  tooltip += i18n("Scheduler priority: %1", process->niceLevel);
 				  break;
 			}
 			if(process->scheduler != KSysGuard::Process::Other)
@@ -875,21 +907,24 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 			return tooltip + "<br />" + tracer;
 		}	
 		case HeadingCPUUsage: {
-			QString tooltip = ki18n("<qt>Process status: %1 %2<br />"
-						"User CPU usage: %3%<br />System CPU usage: %4%</qt>")
+			int divideby = (d->mNormalizeCPUUsage?d->mNumProcessorCores:1);
+			QString tooltip = ki18n("<qt><p style='white-space:pre'>"
+						"Process status: %1 %2<br />"
+						"User CPU usage : %3%<br />"
+						"System CPU usage: %4%</qt>")
 						.subs(process->translatedStatus())
 						.subs(d->getStatusDescription(process->status))
-						.subs((float)(process->userUsage) / d->mNumProcessorCores)
-						.subs((float)(process->sysUsage) / d->mNumProcessorCores)
+						.subs((float)(process->userUsage) / divideby)
+						.subs((float)(process->sysUsage) / divideby)
 						.toString();
 
 			if(process->numChildren > 0) {
 				tooltip += ki18n("<br />Number of children: %1<br />Total User CPU usage: %2%<br />"
 						"Total System CPU usage: %3%<br />Total CPU usage: %4%")
 						.subs(process->numChildren)
-						.subs((float)(process->totalUserUsage)/ d->mNumProcessorCores)
-						.subs((float)(process->totalSysUsage) / d->mNumProcessorCores)
-						.subs((float)(process->totalUserUsage + process->totalSysUsage) / d->mNumProcessorCores)
+						.subs((float)(process->totalUserUsage)/ divideby)
+						.subs((float)(process->totalSysUsage) / divideby)
+						.subs((float)(process->totalUserUsage + process->totalSysUsage) / divideby)
 						.toString();
 			}
 			if(process->userTime > 0) 
@@ -916,7 +951,7 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 			return QVariant();
 		}
 		case HeadingMemory: {
-			QString tooltip = "<qt>";
+			QString tooltip = "<qt><p style='white-space:pre'>";
 			if(process->vmURSS != -1) {
 				//We don't have information about the URSS, so just fallback to RSS
 				if(d->mMemTotal > 0)
@@ -931,9 +966,11 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 			return tooltip;
 		}
 		case HeadingSharedMemory: {
-			if(process->vmURSS == -1)
-				return i18n("<qt>Your system does not seem to have this information for us to read, sorry.</qt>");
-			QString tooltip = "<qt>";
+			QString tooltip = "<qt><p style='white-space:pre'>";
+			if(process->vmURSS == -1) {
+				tooltip += i18n("<qt>Your system does not seem to have this information available to be read.</qt>");
+				return tooltip;
+			}
 			if(d->mMemTotal >0)
 				tooltip += i18n("Shared library memory usage: %1 out of %2  (%3 %)", KGlobal::locale()->formatByteSize((process->vmRSS - process->vmURSS) * 1024), KGlobal::locale()->formatByteSize(d->mMemTotal*1024), (process->vmRSS-process->vmURSS)*100/d->mMemTotal);
 			else
@@ -959,7 +996,7 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 				}
 			}
 			if(!tooltip.isEmpty())
-				return "<qt><ul>" + tooltip + "</ul>";
+				return "<qt><p style='white-space:pre'><ul>" + tooltip + "</ul>";
 			return QVariant(QVariant::String);
 		}
 
@@ -1000,8 +1037,8 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 			//
 			//We subtract the uid to sort ascendingly by that, then subtract the cpu usage to sort by that, then finally subtract the memory
 			
-			long long base = 0;
-			long long memory = 0;
+			qlonglong base = 0;
+			qlonglong memory = 0;
 			if(process->vmURSS != -1) memory = process->vmURSS;
 			else memory = process->vmRSS;
 			if(d->mIsLocalhost && process->uid == getuid())
@@ -1038,17 +1075,81 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 		 }
 		case HeadingMemory:
 			if(process->vmURSS == -1) 
-				return (long long)-process->vmRSS;
+				return (qlonglong)-process->vmRSS;
 			else
-				return (long long)-process->vmURSS;
+				return (qlonglong)-process->vmURSS;
 		case HeadingVmSize:
-			return (long long)-process->vmSize;
+			return (qlonglong)-process->vmSize;
 		case HeadingSharedMemory:
-			if(process->vmURSS == -1) return (long long)0;
-			return (long long)-(process->vmRSS - process->vmURSS);
+			if(process->vmURSS == -1) return (qlonglong)0;
+			return (qlonglong)-(process->vmRSS - process->vmURSS);
 		}
 		return QVariant();
 	}
+	case PlainValueRole:  //Used to return a plain value.  For copying to a clipboard etc
+	{
+		KSysGuard::Process *process = reinterpret_cast< KSysGuard::Process * > (index.internalPointer());
+		switch(index.column()) {
+		case HeadingName:
+			return process->name;
+		case HeadingPid:
+			return (qlonglong)process->pid;
+		case HeadingUser:
+			if(!process->login.isEmpty()) return process->login;
+			if(process->uid == process->euid)
+				return d->getUsernameForUser(process->uid, false);
+			else
+				return d->getUsernameForUser(process->uid, false) + ", " + d->getUsernameForUser(process->euid, false);
+		case HeadingNiceness:
+			return process->niceLevel;
+		case HeadingTty:
+			return process->tty;
+		case HeadingCPUUsage:
+			{
+				double total;
+				if(d->mShowChildTotals && !d->mSimple) total = process->totalUserUsage + process->totalSysUsage;
+				else total = process->userUsage + process->sysUsage;
+
+				if(d->mNormalizeCPUUsage)
+					return total / d->mNumProcessorCores;
+				else
+					return total;
+			}
+		case HeadingMemory:
+			if(process->vmRSS == 0) return QVariant(QVariant::String);
+			if(process->vmURSS == -1) {
+				return (qlonglong)process->vmRSS;
+			} else {
+				return (qlonglong)process->vmURSS;
+			}
+		case HeadingVmSize:
+			return (qlonglong)process->vmSize;
+		case HeadingSharedMemory:
+			if(process->vmRSS - process->vmURSS < 0 || process->vmURSS == -1) return QVariant(QVariant::String);
+			return (qlonglong)(process->vmRSS - process->vmURSS);
+		case HeadingCommand: 
+			{
+				return process->command;
+			}
+#ifdef Q_WS_X11
+		case HeadingXTitle:
+			{
+				if(!d->mPidToWindowInfo.contains(process->pid)) return QVariant(QVariant::String);
+				WindowInfo w = d->mPidToWindowInfo.value(process->pid);
+				if(!w.netWinInfo) return QVariant(QVariant::String);
+				const char *name = w.netWinInfo->visibleName();
+				if( !name || name[0] == 0 )
+					name = w.netWinInfo->name();
+				if(name && name[0] != 0)
+					return QString::fromUtf8(name);
+				return QVariant(QVariant::String);
+			}
+#endif
+		default:
+			return QVariant();
+		}
+		break;
+	}
 #ifdef Q_WS_X11
         case WindowIdRole: {
 		KSysGuard::Process *process = reinterpret_cast< KSysGuard::Process * > (index.internalPointer());
@@ -1081,9 +1182,9 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 		} else if (index.column() == HeadingCPUUsage) {
 			KSysGuard::Process *process = reinterpret_cast< KSysGuard::Process * > (index.internalPointer());
 			if(process->status == KSysGuard::Process::Stopped || process->status == KSysGuard::Process::Zombie) {
-        			QPixmap pix = KIconLoader::global()->loadIcon("button_cancel", KIconLoader::Small,
-			                KIconLoader::SizeSmall, KIconLoader::DefaultState, QStringList(),
-				        0L, true);
+        	//		QPixmap pix = KIconLoader::global()->loadIcon("button_cancel", KIconLoader::Small,
+		//	                KIconLoader::SizeSmall, KIconLoader::DefaultState, QStringList(),
+		//		        0L, true);
 
 			}
 		}
@@ -1122,7 +1223,7 @@ QVariant ProcessModel::data(const QModelIndex &index, int role) const
 	return QVariant(); //never get here, but make compilier happy
 }
 
-bool ProcessModel::hasGUIWindow(long long pid) const
+bool ProcessModel::hasGUIWindow(qlonglong pid) const
 {
 	return d->mPidToWindowInfo.contains(pid);
 }
@@ -1136,9 +1237,9 @@ bool ProcessModel::isLocalhost() const
 void ProcessModel::setupHeader() {
 	QStringList headings;
 	headings << i18nc("process heading", "Name");
-	headings << i18nc("process heading", "User Name");
-	headings << i18nc("process heading", "Pid");
-	headings << i18nc("process heading", "Tty");
+	headings << i18nc("process heading", "Username");
+	headings << i18nc("process heading", "PID");
+	headings << i18nc("process heading", "TTY");
 	headings << i18nc("process heading", "Niceness");
 	// xgettext: no-c-format
 	headings << i18nc("process heading", "CPU %");
@@ -1168,7 +1269,7 @@ void ProcessModel::retranslateUi()
 	setupHeader();
 }
 
-KSysGuard::Process *ProcessModel::getProcess(long long pid) {
+KSysGuard::Process *ProcessModel::getProcess(qlonglong pid) {
 	return d->mProcesses->getProcess(pid);
 }
 
@@ -1195,7 +1296,7 @@ void ProcessModel::setShowTotals(bool showTotals)  //slot
        }
 }
 
-long long ProcessModel::totalMemory() const
+qlonglong ProcessModel::totalMemory() const
 {
 	return d->mMemTotal;
 }
@@ -1208,15 +1309,32 @@ ProcessModel::Units ProcessModel::units() const
 	return (Units) d->mUnits;
 }
 
-QString ProcessModel::formatMemoryInfo(long amountInKB) const
+QString ProcessModel::formatMemoryInfo(qlonglong amountInKB) const
 {
+	//We cache the result of i18n for speed reasons.  We call this function 
+	//hundreds of times, every second or so
+	static QString kbString = i18n("%1 K", QString::fromLatin1("%1"));
+	static QString mbString = i18n("%1 M", QString::fromLatin1("%1"));
+	static QString gbString = i18n("%1 G", QString::fromLatin1("%1"));
+	static QString percentageString = i18n("%1%", QString::fromLatin1("%1"));
+	double amount; 
 	switch(d->mUnits) {
 	  case UnitsKB:
-		return  i18n("%1 k", amountInKB);
+		return kbString.arg(amountInKB);
 	  case UnitsMB:
-		return  i18n("%1 m", (amountInKB+512)/1024);  //Round to nearest megabyte
+		amount = amountInKB/1024.0;
+		if(amount < 0.1) amount = 0.1;
+		return mbString.arg(amount, 0, 'f', 1);
 	  case UnitsGB:
-		return  i18n("%1 g", (amountInKB+512*1024)/(1024*1024)); //Round to nearest gigabyte
+		amount = amountInKB/(1024.0*1024.0);
+		if(amount < 0.1) amount = 0.1;
+		return gbString.arg(amount, 0, 'f', 1);
+	  case UnitsPercentage:
+		if(d->mMemTotal == 0)
+			return ""; //memory total not determined yet.  Shouldn't happen, but don't crash if it does
+		float percentage = amountInKB*100.0/d->mMemTotal;
+		if(percentage < 0.1) percentage = 0.1;
+		return percentageString.arg(percentage, 0, 'f', 1);
 	}
 	return "";  //error
 }
@@ -1224,4 +1342,105 @@ QString ProcessModel::formatMemoryInfo(long amountInKB) const
 QString ProcessModel::hostName() const {
 	return d->mHostName;
 }
+QStringList ProcessModel::mimeTypes() const
+{
+	QStringList types;
+	types << "text/plain";
+	types << "text/csv";
+	types << "text/html";
+	return types;
+}
+QMimeData *ProcessModel::mimeData(const QModelIndexList &indexes) const
+{
+	QMimeData *mimeData = new QMimeData();
+	QString textCsv;
+	QString textCsvHeaders;
+	QString textPlain;
+	QString textPlainHeaders;
+	QString textHtml;
+	QString textHtmlHeaders;
+	QString display;
+	int firstColumn = -1;
+	bool firstrow = true;
+	foreach (const QModelIndex &index, indexes) {
+		if (index.isValid()) {
+			if(firstColumn == -1)
+				firstColumn = index.column();
+			else if(firstColumn != index.column())
+				continue;
+			else {
+				textCsv += '\n';
+				textPlain += '\n';
+				textHtml += "</tr><tr>";
+				firstrow = false;
+			}
+			for(int i = 0; i < d->mHeadings.size(); i++) {
+				if(firstrow) {
+					QString heading = d->mHeadings[i];
+					textHtmlHeaders += "<th>" + heading + "</th>";
+					if(i) {
+						textCsvHeaders += ',';
+						textPlainHeaders += ", ";
+					}
+					textPlainHeaders += heading;
+					heading.replace('"', "\"\"");
+					textCsvHeaders += '"' + heading + '"';
+				}
+				QModelIndex index2 = createIndex(index.row(), i, reinterpret_cast< KSysGuard::Process * > (index.internalPointer()));
+				QString display = data(index2, PlainValueRole).toString();
+				if(i) {
+					textCsv += ',';
+					textPlain += ", ";
+				}
+				textHtml += "<td>" + Qt::escape(display) + "</td>";
+				textPlain += display;
+				display.replace('"',"\"\"");
+				textCsv += '"' + display + '"';
+			}
+		}
+	}
+	textHtml = "<html><table><tr>" + textHtmlHeaders + "</tr><tr>" + textHtml + "</tr></table>";
+	textCsv = textCsvHeaders + '\n' + textCsv;
+	textPlain = textPlainHeaders + '\n' + textPlain;
+
+	mimeData->setText(textPlain);
+	mimeData->setHtml(textHtml);
+	mimeData->setData("text/csv", textCsv.toUtf8());
+	return mimeData;
+
+}
+Qt::ItemFlags ProcessModel::flags(const QModelIndex &index) const
+{
+	Qt::ItemFlags defaultFlags = QAbstractItemModel::flags(index);
+	if (index.isValid())
+		return Qt::ItemIsDragEnabled | defaultFlags;
+	else
+		return defaultFlags;
+
+}
+
+bool ProcessModel::isShowCommandLineOptions() const
+{
+	return d->mShowCommandLineOptions;
+}
+void ProcessModel::setShowCommandLineOptions(bool showCommandLineOptions)
+{
+	d->mShowCommandLineOptions = showCommandLineOptions;
+}
+bool ProcessModel::isShowingTooltips() const
+{
+	return d->mShowingTooltips;
+}
+void ProcessModel::setShowingTooltips(bool showTooltips)
+{
+	d->mShowingTooltips = showTooltips;
+}
+bool ProcessModel::isNormalizedCPUUsage() const
+{
+	return d->mNormalizeCPUUsage;
+}
+void ProcessModel::setNormalizedCPUUsage(bool normalizeCPUUsage)
+{
+	d->mNormalizeCPUUsage = normalizeCPUUsage;
+}
 
