@@ -21,7 +21,7 @@
 
 #include <KGenericFactory>
 
-#include <plasma/abstractrunner.h>
+#include <Plasma/AbstractRunner>
 
 class QWidget;
 
@@ -36,11 +36,13 @@ class CalculatorRunner : public Plasma::AbstractRunner
         CalculatorRunner(QObject* parent, const QVariantList &args);
         ~CalculatorRunner();
 
-        void match(Plasma::SearchContext *context);
-//         bool exec(QAction* action, const QString& term);
+        void match(Plasma::RunnerContext &context);
 
     private:
         QString calculate(const QString& term);
+        void userFriendlySubstitutions(QString& cmd);
+        void powSubstitutions(QString& cmd);
+        void hexSubstitutions(QString& cmd);
 };
 
 K_EXPORT_PLASMA_RUNNER(calculatorrunner, CalculatorRunner)
